package com.dayou.service.impl;

import com.dayou.brokerage.constants.BrokerageBusiness;
import com.dayou.brokerage.constants.BrokerageState;
import com.dayou.common.BaseEntity;
import com.dayou.dto.SalarySettleDTO;
import com.dayou.entity.ItemBrokerageDetail;
import com.dayou.entity.ItemBrokerageGeneral;
import com.dayou.entity.ItemBrokerageSequence;
import com.dayou.entity.ItemBrokerageSequenceDetail;
import com.dayou.mapper.ItemBrokerageSequenceMapper;
import com.dayou.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

/**
 * <p>
 * 项目提成流水 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2023-03-03
 */
@Service
public class ItemBrokerageSequenceServiceImpl extends ServiceImpl<ItemBrokerageSequenceMapper, ItemBrokerageSequence> implements IItemBrokerageSequenceService {

    @Autowired
    private IItemBrokerageSequenceDetailService brokerageSequenceDetailService;

    @Autowired
    private IItemService iItemService;

    @Autowired
    private IItemBrokerageGeneralService brokerageGeneralService;

    @Autowired
    private IItemBrokerageDetailService brokerageDetailService;

    @Override
    @SuppressWarnings("unchecked")
    public Page<ItemBrokerageSequence> selectPage(Page page,ItemBrokerageSequence itemBrokerageSequence){
        return this.page(page, new QueryWrapper<ItemBrokerageSequence>(itemBrokerageSequence));
    }


    @Override
    public ItemBrokerageSequence detail(Long id){
        return this.getById(id);
    }

    @Override
    public Boolean add(ItemBrokerageSequence itemBrokerageSequence){
        return  this.save(itemBrokerageSequence);
    }

    @Override
    public Boolean update(ItemBrokerageSequence itemBrokerageSequence){
        return  this.updateById(itemBrokerageSequence);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Transactional
    @Override
    public Boolean salarySettle() {
        List<ItemBrokerageGeneral> generals = brokerageGeneralService.settleAbleList();
        Map<Long, ItemBrokerageGeneral> idGeneral = generals.stream().collect(Collectors.toMap(BaseEntity::getId, ItemBrokerageGeneral -> ItemBrokerageGeneral));
        if (CollectionUtils.isNotEmpty(generals)){
            Set<Long> generalIds = generals.stream().map(ItemBrokerageGeneral::getId).collect(Collectors.toSet());
            List<ItemBrokerageDetail> settleDetails = brokerageDetailService.list(new LambdaQueryWrapper<ItemBrokerageDetail>()
                    .in(ItemBrokerageDetail::getGeneralId, generalIds).eq(BaseEntity::getDeleted, Boolean.FALSE));
            if (CollectionUtils.isNotEmpty(settleDetails)){
                Set<ItemBrokerageSequenceDetail> sequenceDetails = settleDetails.stream().map(x -> {
                    ItemBrokerageSequenceDetail sequenceDetail = new ItemBrokerageSequenceDetail();
                    sequenceDetail.setBrokerageDetailId(x.getId());
                    sequenceDetail.setUserId(x.getUserId());
                    if (x.getAheadAmount() != null) {
                        sequenceDetail.setBrokerageCate(BrokerageBusiness.AHEAD.getCode());
                        sequenceDetail.setBrokerageAmount(x.getAheadAmount());
                    } else {
                        sequenceDetail.setBrokerageCate(BrokerageBusiness.SETTLE.getCode());
                        sequenceDetail.setBrokerageAmount(x.getActualAmount());
                    }
                    return sequenceDetail;
                }).collect(Collectors.toSet());

                Map<Long, List<ItemBrokerageSequenceDetail>> collect = sequenceDetails.stream().collect(Collectors.groupingBy(ItemBrokerageSequenceDetail::getUserId));

                for (Map.Entry<Long,List<ItemBrokerageSequenceDetail>> userSequence : collect.entrySet()){
                    List<ItemBrokerageSequenceDetail> values = userSequence.getValue();
                    BigDecimal userTotalAmount = values.stream().map(ItemBrokerageSequenceDetail::getBrokerageAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    //todo
                    ItemBrokerageSequence sequence = ItemBrokerageSequence.builder().brokerageAmount(userTotalAmount).userId(userSequence.getKey()).remark("").embody("").build();
                    this.save(sequence);
                    values.stream().forEach(x->{x.setSequenceId(sequence.getId());});
                    brokerageSequenceDetailService.saveBatch(values);
                }

                Map<Long, List<ItemBrokerageDetail>> itemDetails = settleDetails.stream().collect(Collectors.groupingBy(ItemBrokerageDetail::getGeneralId));

                for (Map.Entry<Long,List<ItemBrokerageDetail>> itemDetail : itemDetails.entrySet()){
                    BigDecimal gAheadTotal = BigDecimal.ZERO;
                    BigDecimal gAdvanceTotal = BigDecimal.ZERO;
                    for (ItemBrokerageDetail itemBrokerageDetail :itemDetail.getValue()){
                        //如果预提金额不为空，且已结算金额为空
                        if (itemBrokerageDetail.getAheadAmount().compareTo(BigDecimal.ZERO)!=0 && itemBrokerageDetail.getAdvanceAmount().compareTo(BigDecimal.ZERO) ==0) {
                            gAheadTotal.add(itemBrokerageDetail.getAheadAmount());
                            gAdvanceTotal.add(itemBrokerageDetail.getAheadAmount());
                        }else if (itemBrokerageDetail.getAheadAmount().compareTo(BigDecimal.ZERO)!=0 && itemBrokerageDetail.getAdvanceAmount().compareTo(BigDecimal.ZERO) !=0){
                            gAdvanceTotal.add(itemBrokerageDetail.getActualAmount());
                        }else {
                            continue;
                        }
                    }
                    ItemBrokerageGeneral general = idGeneral.get(itemDetail.getKey());
                    general.setMarketerAdvanceBrokerageDone(gAheadTotal.compareTo(BigDecimal.ZERO)!=0);
                    general.setBrokerageActualAmount(gAdvanceTotal.add(gAheadTotal));
                    if (gAheadTotal.compareTo(BigDecimal.ZERO)==0){
                        //todo
                        general.setStatus(BrokerageState.CLOSED.getCode());
                    }
                }
                brokerageGeneralService.updateBatchById(generals);
            }

        }
        return null;
    }
}
